﻿using System;
using System.Data;
    
using System.Web.Configuration;
using System.Data.SqlClient;
using System.Text;
    
public partial class ExpressionColumnsRelation : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        // Извлекаем строку соединения с именем Northwind из файла web.config
        string connectionString = WebConfigurationManager.
            ConnectionStrings["Northwind"].ConnectionString;
    
        // Создаем объект соединения
        SqlConnection con = new SqlConnection(connectionString);
    
        // Формируем строки SQL-запросов
        string sqlCategories = "SELECT CategoryID, CategoryName FROM Categories";
        string sqlProducts = "SELECT ProductName, CategoryID, UnitPrice FROM Products";
    
        // Создаем объект DataAdapter
        SqlDataAdapter adapter = new SqlDataAdapter(sqlCategories, con);
    
        // Создаем пустой объект DataSet набора данных
        DataSet dataset = new DataSet();
    
        // Выполняем два запроса к БД с открытием 
        // и закрытием соединения вручную.
        // Возможные исключения не обрабатываем, а просто подавляем
        try
        {
            con.Open();
            // Наполнить DataSet данными из таблицы Categories
            // с именованной меткой CatTable
            adapter.Fill(dataset, "CatTable");
            // Сменить команду и добавить в DataSet данные
            // с именованной меткой ProdTable из таблицы Products
            adapter.SelectCommand.CommandText = sqlProducts;
            adapter.Fill(dataset, "ProdTable");
        }
        finally
        {
            con.Close();
        }
    
        // Определение отношения между  извлеченными в DataSet
        // именованными данными CatTable и ProdTable
        DataRelation relation = new DataRelation(
            "CatProd",                                          // Имя отношения
            dataset.Tables["CatTable"].Columns["CategoryID"],   // Родительская таблица
            dataset.Tables["ProdTable"].Columns["CategoryID"]   // Дочерняя таблица
                                                    );
        // Добавление отношения в коллекцию отношений DataSet
        dataset.Relations.Add(relation);
    
        // Создать вычисляемые столбцы и добавить их в DataSet
        DataColumn count = new DataColumn("Кол. продуктов", typeof(int), 
            "COUNT(Child(CatProd).CategoryID)");
        dataset.Tables["CatTable"].Columns.Add(count);
        DataColumn max = new DataColumn("Самый дорогой продукт", typeof(decimal),
            "MAX(Child(CatProd).UnitPrice)");
        dataset.Tables["CatTable"].Columns.Add(max);
        DataColumn min = new DataColumn("Самый дешевый продукт", typeof(decimal),
            "MIN(Child(CatProd).UnitPrice)");
        dataset.Tables["CatTable"].Columns.Add(min);

        // Меняем названия заголовков столбцов на русский язык
        dataset.Tables["CatTable"].Columns["CategoryID"].ColumnName = "№ п/п";
        dataset.Tables["CatTable"].Columns["CategoryName"].ColumnName = "Наименование категории";
    
        // Показываем данные пользователю
        GridView1.DataSource = dataset.Tables["CatTable"];
        GridView1.DataBind();
    }
}